/***************************************************************
*
* Copyright (C) 2008-2010, Distributed Systems Laboratory,
* Center for Computation & Technology, and Department of Computer Science
* Louisiana State University, Baton Rouge, LA.
*
* Licensed under the Apache License, Version 2.0 (the "License"); you
* may not use this file except in compliance with the License.  You may
* obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
***************************************************************/

/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "condor_common.h"
#include "dap_constants.h"
#include "dap_utility.h"
#include "dap_error.h"
#include "dap_logger.h"
#include <unistd.h>
//---
#include "dap_url.h"

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <string.h>

//===

//---

#define MAX_STR 512


/* ============================================================================
 * Mkdir at local filesystem
 * ==========================================================================*/

int file_mkdir(char *dst, int mymode){
  char *p = dst;

  do{
    p = strchr(p+1, '/');
    if(p) *p = '\0';

    // errno == 17 means that the directory already exists
    // we therefore ignore this error
    if(mkdir(dst, mymode) == -1 && errno != 17){
      fprintf(stderr, "mkdir %s failed: %s %d\n", dst, strerror(errno), errno);
      return -1;
    }
    if(p) *p = '/';
  }while(p);

  return 0;
}

/*===================================================================
  Setting stork specific parameters (filesize verify, chksum verify)

  ===================================================================
*/


int set_stork_parameters(int *filesizeVerify, int *checksumVerify, int *mymode){

  char *envVal;

  /* Set file permissions as provided */
  if ((envVal=getenv("STORK_SET_PERMISSIONS")) != NULL){
  //if ((envVal=myEnv.GetEnv("STORK_SET_PERMISSIONS")) != NULL){
    *mymode = atoi(envVal);
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_FILESIZE"))&&(!strncasecmp("TRUE",envVal,4))){
  //if ((envVal=myEnv.GetEnv("STORK_VERIFY_FILESIZE"))
		//&&(!strncasecmp("TRUE",envVal,4))){
    *filesizeVerify = 1;
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_CHECKSUM"))&&(!strncasecmp("TRUE",envVal,4))){
  //if ((envVal=myEnv.GetEnv("STORK_VERIFY_CHECKSUM"))
		//&&(!strncasecmp("TRUE",envVal,4))){
    *checksumVerify = 1;
  }
 
  
  return 0;
}
//===

/* ============================================================================
 * Transfer a file from local disk to local disk
 * ==========================================================================*/
int transfer_from_file_to_file(char *src_file, char *dest_file,
				int filesizeVerify, int checksumVerify, int mymode)
{
	int in_fd, out_fd;
    char *buf;
    long src_filesize, dest_filesize;
    int bytes_read, bytes_written;
    //struct stat src_filestat, dest_filestat;
//---
	struct stat64 src_filestat, dest_filestat;
	char localchksum[10], localchksum2[10];
	char chksumbuf[10 + strlen(src_file)], chksumbuf2[10 + strlen(dest_file)];
	FILE *p_fd, *p_fd2;
//===

    //start transfer
    buf = (char*)malloc(sizeof(char) * BUF_SIZE);
    
    //get the src filesize info
    //if (!lstat(src_file, &src_filestat))  //if file exists
//---
    if (!lstat64(src_file, &src_filestat))	
//===
      src_filesize = src_filestat.st_size;

    //open the input file for reading
    //in_fd = safe_open_wrapper(src_file, O_RDONLY);
	in_fd = open(src_file, O_RDONLY);

    if (in_fd < 0){
      fprintf(stdout,"Error in opening file : %s\n",src_file);
      free(buf);
      return DAP_ERROR;
    }
    
/*
    //get dest directory name
    char tmpstr[MAXSTR], dir[MAXSTR], dest_fname[MAXSTR];
    char *t = NULL, *p = NULL;

    strcpy(tmpstr, dest_file);
    strcpy(dir, "");

    t = strtok(tmpstr,"/");
//---
	//printf("test t=%s\n", t);
//===
    while(1){
      if (t == NULL) break;
      if (p!= NULL) strcat(strcat(dir, "/"), p);
      if ((p = strtok(NULL,"/")) != NULL){
	strcat(strcat(dir, "/"), t);   //get file name
      }
      else {
	strcpy(dest_fname, t);
	break;
      }
      t = strtok(NULL,"/");
    }

    if (t == NULL){
      if (p != NULL) 
	strcpy(dest_fname, p);
      else
	strcpy(dest_fname, "");
    }
    
    if (!strcmp(dir, "")) 
      strcpy (dir,"/");
    
    //check if dest directory exists, if not create
    struct stat dest_dirstat;
    if (lstat(dir, &dest_dirstat)){ //if dest dir does not exist
      char command[MAXSTR];
      
      snprintf(command, MAXSTR, "mkdir -p %s", dir);
      if (system(command) == -1){
	fprintf(stdout,"Error in creating directory : %s\n",dir);
	return DAP_ERROR;
      }
    }
*/

    //open the output file for writing
    //out_fd = safe_open_wrapper(dest_file, O_CREAT | O_RDWR, 00777);
    out_fd =  open(dest_file, O_CREAT | O_RDWR, 00777);

    if (out_fd < 0){
      fprintf(stdout,"Error in opening file : %s\n",dest_file);
      free(buf);
      close(in_fd);
      return DAP_ERROR;
    }

    //transfer file
    while ( (bytes_read = read(in_fd, buf, BUF_SIZE)) > 0){
      bytes_written = write(out_fd, buf, bytes_read);
      
      if (bytes_written != bytes_read){
	fprintf(stdout,"Error! read = %d, written = %d\n",bytes_read, bytes_written);
	free(buf);
	close(in_fd);
	close(out_fd);
	return DAP_ERROR;
      }
    }

    //get the dest filesize info
    //if (!lstat(dest_file, &dest_filestat))  //if file exists
//---
    if (!lstat64(dest_file, &dest_filestat))
//===
      	dest_filesize = dest_filestat.st_size;

     //file size verification
     if(filesizeVerify) {
	//verify source and destination file sizes
    	if (src_filesize != dest_filesize){
      		printf("src and dest file sizes do not match!!\n");
      		free(buf);
      		close(in_fd);
      		close(out_fd);
      		return DAP_ERROR;
    	} 
      }
     
     //check sum verification
     if(checksumVerify){
	//caculate the checksum for src file and dest file
	sprintf(chksumbuf, "sum -s %s", src_file);
	sprintf(chksumbuf2, "sum -s %s", dest_file);
	
	p_fd = popen(chksumbuf,"r");
	p_fd2 = popen(chksumbuf2, "r");
		
	if (p_fd == NULL || p_fd2 == NULL){
	    fprintf(stderr, "Error: unable to execute chksum\n");
	    return -1;	
	}
	
	fscanf(p_fd, "%s", localchksum);
	fscanf(p_fd2, "%s", localchksum2);
	pclose(p_fd);
	pclose(p_fd2);
	
	//verify source and destination checksum values
	if (atoi(localchksum) != atoi(localchksum2)){
		fprintf(stderr, "Error: checksum mismatch src: %d dest: %d\n",
			atoi(localchksum), atoi(localchksum2));
		fprintf(stderr, "Error: deleting the dest file");
		unlink(dest_file);
		return -1;
	}
     }	

    //otherwise file is transfered succesfully..
    printf("SUCCESS!\n");
    free(buf);
    close(in_fd);
    close(out_fd);
    return DAP_SUCCESS;

}

//--
/* =============================================================
	performing transfers from local filesystem, to iRODS
	parse submitted URLS
	Obtain Stork Parameters
	Perform transfers according to type
	  recursive
	  wildcard
	  single file to file object
	  else print error message
   =================================================================
*/

int file_to_file(char *src, char *dst){

	int status;
	int checksumVerify =0, filesizeVerify = 0, mymode = 0700;	

	//--------Parsing URLs-----------------------
	//stork parse srouce url
	Url fileParsed(src);
	if(!fileParsed.parsed()){
		fprintf(stderr, "parse src URL '%s' error\n",
			fileParsed.urlNoPassword());	
		return -1;
	}

	//stork parse destination url;
	Url fileParsed2(dst);
        if(!fileParsed2.parsed()){
                fprintf(stderr, "parse dest URL '%s' error\n",
                        fileParsed2.urlNoPassword());
                return -1;
        }

	if (fileParsed.path()==NULL || fileParsed2.path() == NULL){
		fprintf(stderr, "Error: some fields are missing,"
			"please recheck your URL and resubmit the job.\n"
			"src:%s\ndest:%s\n", fileParsed.urlNoPassword(),
					     fileParsed2.urlNoPassword());
		return -1;
	}

/*
	printf("src dir = %s\n", fileParsed.dirname());
	printf("src filename = %s\n", fileParsed.basename());

	printf("dest dir = %s\n", fileParsed2.dirname());
	printf("dest filename = %s\n", fileParsed2.basename());
*/	
	
	//------------------Obtain Stork Parameters-----------------------

	set_stork_parameters(&filesizeVerify, &checksumVerify, &mymode);

	//====================== Recursive Transfer ==========================
	if (fileParsed.dirname()!=NULL && fileParsed2.dirname() != NULL &&
	    fileParsed.basename() == NULL && fileParsed2.basename() == NULL){
		
		//Perform recursive copy of the directory given in the source url
		FILE *infile;
		char path[MAX_STR];
		char dest[MAX_STR];
		char cmd[MAX_STR];

		//Create required directories
		sprintf(cmd, "find %s -type d", fileParsed.dirname());
		infile = popen(cmd, "r");
		if (infile == NULL){
			fprintf(stderr, "Error: Unable to open: tmp-filelist\n");
			return -1;
		}

		while(fgets(path, MAX_STR, infile) != NULL){
			strcpy(path + strlen(path) -1, "\0");
			sprintf(dest, "%s%s", fileParsed2.dirname(),
						path+strlen(fileParsed.dirname()));
			
			status = file_mkdir(dest, mymode);
			if (status < 0){
			    fprintf(stderr,"Collection creation error, status=%d\n",
						status);
				return -1;
			}
		}//while
		fclose(infile);
		//finished creating required directories

		//List all files in the directory recursively and transfer
		sprintf(cmd, "find %s -type f", fileParsed.dirname());
		infile = popen(cmd, "r");

		if (infile == NULL){
		    fprintf(stderr, "Error: Unable to open tmp-filelist\n");	
			return -1;
		}
		
		while(fgets(path, MAX_STR, infile) != NULL){
			strcpy(path+strlen(path) -1, "\0");
			sprintf(dest, "%s%s", fileParsed2.dirname(),
						path+strlen(fileParsed.dirname()));

			status = transfer_from_file_to_file(path, dest, 
						  filesizeVerify, checksumVerify, mymode); 
			//mymode should be added
			if (status < 0){
				fprintf(stderr, "Error: transfer file"
					" %s to file %s failed!\n",
						path, dest);
				return -1;
				
			}
		}
		fclose(infile);
	} //if 
	//===============================wild card========================================
	else if (fileParsed.basename()!=NULL && fileParsed2.basename()==NULL &&
			strchr(fileParsed.basename(),'*') != NULL){

	  FILE * infile;
	  char path[MAX_STR];
          char dest[MAX_STR];
          char cmd[MAX_STR];

	  status = file_mkdir(fileParsed2.dirname(), mymode);
	  if (status < 0){
		fprintf(stderr, "Collection creation error, status =%d\n",status);
		return -1;
	  }
	  
          sprintf(cmd, "find %s -maxdepth 1 -name \"%s\" -type f", 
			fileParsed.dirname(), fileParsed.basename());
	  infile = popen(cmd, "r");
	  if(infile == NULL){
		fprintf(stderr, "Error, Unable to open pipe:\n");
		return -1;
	  }
	
	  while(fgets(path, MAX_STR, infile) != NULL){
		strcpy(path+strlen(path)-1, "\0");
		sprintf(dest, "%s%s", fileParsed2.dirname(), strrchr(path, '/'));
		status = transfer_from_file_to_file(path, dest,	
					filesizeVerify, checksumVerify, mymode);

		if (status <0){
			fprintf(stderr, "Error: transfer" 
				"file %s to file %s failed\n",
				path, dest);
			return -1;
		}
	  }
	  fclose(infile);

	}//else if
	//======================== single file =======================================
	else if (fileParsed.basename()!=NULL && 
		(strchr(fileParsed.basename(),'*') == NULL)){

		char dest[MAX_STR];
		int len = strlen(fileParsed2.path());
		strncpy(dest, fileParsed2.path(), len+1);

		//create required directories
		status = file_mkdir(fileParsed2.dirname(), mymode);
		
		if (status < 0){
		  fprintf(stderr, "Collection creation error, status = %d\n", status);
		   return -1;
		}
		
		//use source filename as desination filename 
		//if destination file name not given
		if (fileParsed2.basename() == NULL){
		    //len = strlen(fileParsed2.path())+ strlen(fileParsed.basename());
		    sprintf(dest, "%s/%s", fileParsed2.dirname(), 
                                           fileParsed.basename());
		}

		//call to transfer function
		status = transfer_from_file_to_file(fileParsed.path(), dest,
						filesizeVerify, checksumVerify, mymode);

		if (status < 0){
		    fprintf(stderr, "Error: transfer file %s to file % s failed\n",
				fileParsed.path(), dest);
		    return -1;
		}
	}
	else {
		fprintf(stderr, "please check the submitted urls and try again\n");
		fprintf(stderr, "src:%s dest:%s\n", fileParsed.urlNoPassword(),
						    fileParsed2.urlNoPassword());
		return -1;
	}	

	return 0;
}
//===

int main(int argc, char *argv[])
{

/*
  char src_protocol[MAXSTR], src_host[MAXSTR], src_file[MAXSTR];
  char dest_protocol[MAXSTR], dest_host[MAXSTR], dest_file[MAXSTR];
  char src_url[MAXSTR], dest_url[MAXSTR];

  int status = 0;
*/
//---
  int status;
//===

  if (argc < 3){
    fprintf(stderr, "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url>\n", argv[0]);
    fprintf(stderr, "==============================================================\n");
    exit(-1);
  }
/*
  strncpy(src_url, argv[1], MAXSTR-1);
  strncpy(dest_url, argv[2], MAXSTR-1);
 
  parse_url(src_url, src_protocol, src_host, src_file);
  parse_url(dest_url, dest_protocol, dest_host, dest_file);
  
  fprintf(stdout, "Transfering file:%s from host %s to file:%s on host %s..\n",
	  src_file, src_host, dest_file, dest_host);

  if ( !strcmp(src_protocol, "file") && !strcmp(dest_protocol, "file")){
    status = transfer_from_file_to_file(src_file, dest_file);
  }
*/
  
//---
	if (!strncmp(argv[1], "file:/",6) && !strncmp(argv[2], "file:/", 6)){
		status = file_to_file(argv[1], argv[2]);
		if (status<0){
			fprintf(stderr, "Error: could not transfer file to file\n");
			return -1;
		}
	} else {
		printf("please check the syntax of the file jobs submitted");
	}
//===

  return status;
  
}





