/***************************************************************
*
* Copyright (C) 2008-2010, Distributed Systems Laboratory,
* Center for Computation & Technology, and Department of Computer Science
* Louisiana State University, Baton Rouge, LA.
*
* Licensed under the Apache License, Version 2.0 (the "License"); you
* may not use this file except in compliance with the License.  You may
* obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
***************************************************************/

/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 *
 * Condor Software Copyright Notice
 * Copyright (C) 1990-2006, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 *
 * This source code is covered by the Condor Public License, which can
 * be found in the accompanying LICENSE.TXT file, or online at
 * www.condorproject.org.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * AND THE UNIVERSITY OF WISCONSIN-MADISON "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS
 * FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY OF WISCONSIN-MADISON
 * MAKE NO MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS,
 * ENHANCEMENTS OR DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY
 * PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER PROPRIETARY
 * RIGHT.
 *
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

//---
//#include "condor_common.h"
//#include "env.h"
//===

//---
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <string.h>
//===

#include "rodsClient.h" 
#include "rodsPath.h" 
#include "miscUtil.h" 
#include "rodsLog.h"
#include "rodsErrorTable.h"
#include "dap_url.h"
#include "dap_error.h"
#include "dap_constants.h"

//---
#include "rods.h"
#include "rodsGenQuery.h"

#include <sys/time.h>
#include <unistd.h>
//===

#define BUFSIZE	(1024*1024)
#define MAX_STR 512


//---

char STORK_DAPID[MAX_STR] = "";  //new version does not seem like have this
rodsRestart_t rodsRestart;

//---
char file_ckpt_str[] = "stork_file.rescue.";
char file_ckpt_str2[] = "stork_irodsfile.rescue.";
//strcat(file_ckpt_str, getenv("STORK_JOBID"));

//===

//===


/* ============================================================================
 * Create a connection to iRODS
 * ==========================================================================*/

rcComm_t* connect_to_irods(Url& irodsParsed){

  rcComm_t *conn;
  rErrMsg_t errMsg;
  int status;

  // Connecting to iRODS
  conn = rcConnect(irodsParsed.host(), atoi(irodsParsed.port()), 
      irodsParsed.srbUser(), irodsParsed.srbMdasDomain(), 0, &errMsg);
  if (conn == NULL) {
    fprintf (stderr, "rcConnect error %s\n", errMsg.msg);
    exit(-1);
  }

  // Creating a client login session
  status = clientLoginWithPassword(conn, irodsParsed.password());
  if (status != 0) {
    rodsLogError (LOG_ERROR, status,"", "");
    fprintf (stderr, "clientLogin error\n");
    rcDisconnect(conn);
    exit(-1);
  }

  return conn;

}


/* ============================================================================
 * Mkdir at local filesystem
 * ==========================================================================*/

int file_mkdir(char *dst, int mymode){
  char *p = dst;

  do{
    p = strchr(p+1, '/');
    if(p) *p = '\0';
    
    // errno == 17 means that the directory already exists
    // we therefore ignore this error
    if(mkdir(dst, mymode) == -1 && errno != 17){
      fprintf(stderr, "mkdir %s failed: %s %d\n", dst, strerror(errno), errno);
      return -1;
    }
    if(p) *p = '/';
  }while(p);

  return 0;
}

//---

int initCondForChkPnt(rodsRestart_t *rodsRestart){

    rodsArguments_t *rodsArgs;

    memset (rodsRestart, 0, sizeof (rodsRestart_t));
    int status;

    if (getenv("STORK_JOBID"))
    	strcat(file_ckpt_str, getenv("STORK_JOBID"));
    
    //status = openRestartFile (rodsArgs->restartFileString, rodsRestart, rodsArgs);
    status = openRestartFile(file_ckpt_str, rodsRestart, rodsArgs);
    //printf("test1...\n");
    if (status < 0) {
        //rodsLogError (LOG_ERROR, status, "initCondForPut: openRestartFile of %s error", rodsArgs->restartFileString);
        rodsLogError (LOG_ERROR, status, "initCondForPut: openRestartFile of %s error", file_ckpt_str);
        return (status);
    }
    return (0);
}


int convertCharStr(char str1[], char *str2){

    //printf("strlen = %d\n", strlen(str1));
    int i = 0;
    while(*str2!= '\0' && i< MAX_NAME_LEN){
          str1[i] = *str2;
          //printf("%c\n", str1[i]);
          str2++;
          i++;
    }
}

//===



/* ============================================================================
 * Setting Stork specific parameters (filesize verify, chksum verify)
 * ==========================================================================*/

//int set_stork_parameters(int *filesizeVerify, int *checksumVerify, int *mymode, int *nthreads){
int set_stork_parameters(int *filesizeVerify, int *checksumVerify, int *checkPoint, int *fileCKPT, int *mymode, int *nthreads){

  char *envVal;

//---
	//const char *envVal;
	//Env myEnv;
//===

  /* Set file permissions as provided */
  if ((envVal=getenv("STORK_SET_PERMISSIONS")) != NULL){
  //if ((envVal=myEnv.GetEnv("STORK_SET_PERMISSIONS")) != NULL){
    *mymode = atoi(envVal);
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_FILESIZE"))&&(!strncasecmp("TRUE",envVal,4))){
  //if ((envVal=myEnv.GetEnv("STORK_VERIFY_FILESIZE"))&&(!strncasecmp("TRUE",envVal,4))){
    *filesizeVerify = 1;
  }

  /* Perform checksum verification after data transfer */
  if ((envVal=getenv("STORK_VERIFY_CHECKSUM"))&&(!strncasecmp("TRUE",envVal,4))){
  //if ((envVal=myEnv.GetEnv("STORK_VERIFY_CHECKSUM"))&&(!strncasecmp("TRUE",envVal,4))){
    *checksumVerify = 1;
  }

//---

  if((envVal=getenv("STORK_CHECKPOINT_TRANSFER")) && (!strncasecmp("TRUE", envVal, 4))){
  //if((envVal=myEnv.GetEnv("STORK_CHECKPOINT_TRANSFER")) && (!strncasecmp("TRUE", envVal, 4))){
    	*checkPoint = 1;     
    	initCondForChkPnt(&rodsRestart);
    	//setStateForRestart (conn, &rodsRestart, targPath, myRodsArgs);
//---
/*
        int fd_test;
        char buf[100];
        strcpy(buf, envVal);
        if ( (fd_test = open("test_file3.txt", O_TRUNC|O_RDWR|O_CREAT,00700)) == -1){
                        perror("cant open the file");
                        return 1;
        }
        int size = write(fd_test, buf, 50);
        close(fd_test);
*/
//===
  }

   if((envVal=getenv("STORK_FILE_CKPT")) && (!strncasecmp("TRUE", envVal, 4))){
   //if((envVal=myEnv.GetEnv("STORK_FILE_CKPT")) && (!strncasecmp("TRUE", envVal, 4))){
	*fileCKPT = 1;		
   }

/*
   if(envVal=getenv("STORK_JOBID")){
   //if(envVal=myEnv.GetEnv("STORK_DAPID")){
     	strcpy(STORK_DAPID, envVal);
   }
*/


//===


  /* Setting the number of threads for data transfer */
  if( *nthreads < 0){
    if((envVal=getenv("STORK_SET_PARALLELISM")) != NULL){
      //strcpy(STORK_SET_PERMISSIONS,envVal);
//---
/*
	FILE *file_test1 = fopen("test_parallel.txt", "w");
	fprintf(file_test1, "%d\n", atoi(envVal));
	fclose(file_test1);
*/
//===
      *nthreads = atoi(envVal);
      if (*nthreads <= 0 || *nthreads == 1){
        *nthreads = NO_THREADING;
      }
    }else{
      *nthreads = 4;
    }
  }else if(*nthreads <= 0 || *nthreads == 1){
    *nthreads = NO_THREADING;
  }

  //fprintf(stdout, "f: %d c: %d m: %d n: %d\n", 
  //    *filesizeVerify, *checksumVerify, *mymode, *nthreads);

  return 0;

}


/* ============================================================================
 * Transfer a file from local disk to iRODS
 * ==========================================================================*/

/*
int transfer_from_file_to_irods(rcComm_t *conn, char *src, char *dst,
    				int filesizeVerify, int checksumVerify, int mymode, int nthreads){
*/

//---
int transfer_from_file_to_irods(rcComm_t *conn, char *src, char *dst,
                                int filesizeVerify, int checksumVerify,  int fileCKPT,
				int mymode, int nthreads) {
//===

  int status;
  dataObjInp_t dataObjOprInp;
  struct stat64 srcStat;
  rodsObjStat_t *destStat = NULL;
  //char *chksumStr;


  memset (&dataObjOprInp, 0, sizeof (dataObjOprInp));
  snprintf (dataObjOprInp.objPath, MAX_NAME_LEN, "%s", dst);

  addKeyVal (&dataObjOprInp.condInput, DATA_TYPE_KW, "generic");
  addKeyVal (&dataObjOprInp.condInput, FORCE_FLAG_KW, "");

  dataObjOprInp.createMode = mymode;
  //dataObjOprInp.openFlags = O_WRONLY;
  dataObjOprInp.openFlags = O_RDWR;
  dataObjOprInp.numThreads = nthreads;
  //dataObjOprInp.dataSize = srcStat.st_size;


  if(checksumVerify){

    //fprintf(stdout, "Not performing filesize verification\n");

    addKeyVal (&dataObjOprInp.condInput, VERIFY_CHKSUM_KW, "");

    status = rcChksumLocFile (src, VERIFY_CHKSUM_KW, 
      &dataObjOprInp.condInput);
    if (status < 0) {
      fprintf (stderr, "rcChksumLocFile error, status = status %d\n", status);
      return -1;
    }
  }


//---
	if (fileCKPT){
   		 if (getenv("STORK_JOBID"))
        		strcat(file_ckpt_str2, getenv("STORK_JOBID"));
				
		//need to disable multi file ckpt at this time
  		rodsFileRestart_t rodsFileRestart;
     		rodsArguments_t *rodsArgs;

     		memset(&rodsFileRestart, 0, sizeof(rodsFileRestart_t));
       		dataObjOprInp.numThreads = NO_THREADING; // or dataObjOprInp->numThreads = 0;
       		int status1;
       		status1 = openFilerestartFile (file_ckpt_str2, &rodsFileRestart, rodsArgs);

       		char fileRestartOffset[50];
       		sprintf(fileRestartOffset, "%d", rodsFileRestart.bytesCnt);
       		//printf("fileRestartOffset=%s\n",fileRestartOffset);
       		addKeyVal (&dataObjOprInp.condInput, FILERESTART_FLAG_KW, fileRestartOffset);
		
       		if(status1 < 0){
          		rodsLogError (LOG_ERROR, status, "initCondForPut: openFilerestartFile of %s error", file_ckpt_str2);
          		return (status1);
       		}

		status = rcDataObjPut1 (conn, &dataObjOprInp, src, &rodsFileRestart);
	} else {
//===
  		status = rcDataObjPut (conn, &dataObjOprInp, src);
	}


  if (status < 0) {
    rodsLogError (LOG_ERROR, status,"", "");
    fprintf (stderr, "rcDataObjPut error. status = %d\n", status);
    return -1;
  }


  // test rcDataObjStat //
  if(filesizeVerify){
    fprintf(stdout, "Performing filesize verification\n");

    status = lstat64(src, &srcStat);
    if (status < 0) {
      fprintf (stderr, "lstat64 of %s failed\n", src);
      return -1;
    }

    status = rcObjStat(conn, &dataObjOprInp, &destStat);
    if (status < 0) {
      fprintf (stderr, "rcDataObjStat64 error for %s status = status%d\n",
          dataObjOprInp.objPath,status);
      return -1;
    }

    //Verifying source and destination file sizes
    if(((srcStat.st_mode & S_IFREG) != 0) && 
        ((destStat->objType == DATA_OBJ_T) != 0)){

      if ((unsigned long)srcStat.st_size != (unsigned long)destStat->objSize){
        fprintf(stderr, "Error: filesizes differ src: %lu dest: %lu.\n",
            (unsigned long)srcStat.st_size, (unsigned long)destStat->objSize);
        return -1;
      }
    }
  }else{
    //fprintf(stdout, "Not performing filesize verification\n");
  }

  // test rcDataObjChksum //
  /*
  if(checksumVerify){
    fprintf(stdout, "Not performing filesize verification");

    status = rcDataObjChksum (conn, &dataObjOprInp, &chksumStr);
    if (status < 0) {
      fprintf (stderr, "rcDataObjChksum error, status = status%d\n", status);
      return -1;
    }
  }*/

  return 0;

}


/* ============================================================================
 * Transfer an iRODS file to local disk
 * ==========================================================================*/

/*
int transfer_from_irods_to_file(rcComm_t *conn, char *src, char *dst, 
    int filesizeVerify, int checksumVerify, int mymode, int nthreads){
*/
//---
int transfer_from_irods_to_file(rcComm_t *conn, char *src, char *dst, 
    				int filesizeVerify, int checksumVerify, int fileCKPT,
				int mymode, int nthreads){
//===

  dataObjInp_t dataObjOprInp;

  char *chksumStr;
  rodsObjStat_t *srcStat = NULL;
  struct stat64 destStat;
  int status;

  // test rcDataObjGet //
  memset (&dataObjOprInp, 0, sizeof (dataObjOprInp));
  snprintf (dataObjOprInp.objPath, MAX_NAME_LEN, "%s", src);

  //addKeyVal (&dataObjOprInp.condInput, RESC_NAME_KW, RESC_NAME);
  addKeyVal (&dataObjOprInp.condInput, FORCE_FLAG_KW, "");
  addKeyVal (&dataObjOprInp.condInput, DATA_TYPE_KW, "generic");

  if(checksumVerify){
    addKeyVal (&dataObjOprInp.condInput, VERIFY_CHKSUM_KW, "");
  }

  //dataObjOprInp.openFlags = O_RDONLY;
  dataObjOprInp.openFlags = O_RDWR;
  dataObjOprInp.numThreads = nthreads;
  //dataObjOprInp.dataSize = srcStat->objSize;

//---
  if (fileCKPT){
	    if (getenv("STORK_JOBID"))
                strcat(file_ckpt_str2, getenv("STORK_JOBID"));
		
	    rodsFileRestart_t rodsFileRestart;	
	    rodsArguments_t *rodsArgs;

	    memset(&rodsFileRestart, 0, sizeof(rodsFileRestart_t));
    		
           //addKeyVal (&dataObjOprInp->condInput, FILERESTART_FLAG_KW, ""); // addKeyVal (&dataObjOprInp->condInput, FORCE_FLAG_KW, "");
            dataObjOprInp.numThreads = NO_THREADING; // or dataObjOprInp->numThreads = 0;
            int status1;
            char fileRestartOffset[50];
            status1 = openFilerestartFile (file_ckpt_str2, &rodsFileRestart, rodsArgs);
            sprintf(fileRestartOffset, "%d", rodsFileRestart.bytesCnt);
            addKeyVal (&dataObjOprInp.condInput, FILERESTART_FLAG_KW, fileRestartOffset);
            if(status1 < 0){
               rodsLogError (LOG_ERROR, status, "initCondForPut: openFilerestartFile of %s error", file_ckpt_str2);
               return (status1);
            }
		
	    status = rcDataObjGet1 (conn, &dataObjOprInp, dst, &rodsFileRestart);	
  } else{
//===
  	status = rcDataObjGet(conn, &dataObjOprInp, dst);
  }

  if (status < 0) {
    rodsLogError (LOG_ERROR, status,"", "");
    fprintf (stderr, "rcDataObjGet error. status = %d\n", status);
    return -1;
  }

  if(filesizeVerify){

    //destination file size
    if (lstat64(dst, &destStat) < 0){
      fprintf (stderr, "lstat64 of %s failed\n", dst);
      return -1;
    }

    status = rcObjStat(conn, &dataObjOprInp, &srcStat);
    if (status < 0) {
      fprintf (stderr, "rcDataObjStat64 error for %s status = status%d\n",
          dataObjOprInp.objPath,status);
      return -1;
    }

    //Verifying source and destination file sizes
    if( ((srcStat->objType == DATA_OBJ_T) != 0) && 
        (destStat.st_mode & S_IFREG) != 0){
      if ((unsigned long)srcStat->objSize != (unsigned long)destStat.st_size){
        fprintf(stderr, "Error: filesizes differ src: %lu dest: %lu.\n",
            (unsigned long)srcStat->objSize, (unsigned long)destStat.st_size);
        return -1;
      }
    }
  }


  if(checksumVerify){
    status = rcChksumLocFile (dst, VERIFY_CHKSUM_KW, 
        &dataObjOprInp.condInput);
    if (status < 0) {
      fprintf (stderr, "rcChksumLocFile error, status = status %d\n", status);
      return -1;
    }

    // test rcDataObjChksum //
    status = rcDataObjChksum (conn, &dataObjOprInp, &chksumStr);
    if (status < 0) {
      fprintf (stderr, "rcDataObjChksum error, status = status%d\n", status);
      return -1;
    }
  }

  return 0;
}


/* ============================================================================
 * Performing transfers from local filesystem to iRODS
 * Parse submitted URLs
 * Obtain Stork Parameters
 * Connect to SRB server
 * Perform transfers according to type
 *  - Recursive
 *  - wildcard
 *  - single file to data object
 *  - else print error message
 * ==========================================================================*/


int file_to_irods(char *src, char*dst, int nthreads){

  rcComm_t *conn;
  int status;

  int checksumVerify = 0, filesizeVerify = 0, mymode = 0700;

//---
  int checkPoint = 0;
  int fileCKPT = 0;

    //rodsPath_t *targPath;
    //rodsArguments_t *rodsArgs;    
    //dataObjInp_t *dataObjOprInp;

    rodsPath_t targPath;
    rodsArguments_t rodsArgs;
    dataObjInp_t dataObjOprInp;

    memset(&targPath, 0, sizeof(rodsPath_t));
    memset(&rodsArgs, 0, sizeof(rodsArguments_t));
    memset(&dataObjOprInp, 0, sizeof(dataObjInp_t));

    //targPath->outPath = irodsParsed.dirname();
/*
    convertCharStr(targPath.outPath, irodsParsed.dirname());
    rodsArgs.verbose = False;
    dataObjOprInp.oprType = PUT_OPR;

    setStateForRestart (conn, &rodsRestart, &targPath, &rodsArgs);
*/

//===


  //------------------ Parsing URLs -----------------------------------------
  // Stork parse source url
  Url fileParsed(src);
  if(!fileParsed.parsed() ) {
    fprintf(stderr, "parse src URL '%s' error\n",
        fileParsed.urlNoPassword()
        );
    return -1;
  }

  // Stork parse destination url
  Url irodsParsed(dst);
  if(!irodsParsed.parsed() ) {
    fprintf(stderr, "parse dest URL '%s' error\n",
        irodsParsed.urlNoPassword()
        );
    return -1;
  }
 
  // Stork parse the destination url as SRB
  // iRODS has similar fields as SRB
  irodsParsed.parseSrb();
  if(!irodsParsed.parsedSrb() ) {
    fprintf(stderr, "parse dest URL SRB fields'%s' error\n",
        irodsParsed.urlNoPassword());
    return -1;
  }

  if(fileParsed.path() == NULL || irodsParsed.path() == NULL ||
      irodsParsed.host() == NULL || irodsParsed.port() == NULL ||
      irodsParsed.srbUser() == NULL || irodsParsed.srbMdasDomain() == NULL ||
      irodsParsed.password() == NULL){
    fprintf(stderr, "Error: Some fields are missing. Please recheck your URL "\
      "and resubmit the job.\nsrc: %s\ndest: %s\n", fileParsed.urlNoPassword(),
      irodsParsed.urlNoPassword());
    return -1;
  }
  //------------------ Finished Parsing URLs ---------------------------------



  //------------------ Obtain Stork Parameters -------------------------------

  //set_stork_parameters(&filesizeVerify, &checksumVerify, &mymode, &nthreads);
  set_stork_parameters(&filesizeVerify, &checksumVerify, &checkPoint, &fileCKPT, &mymode, &nthreads);

  //------------------ Finish obtaining Stork Parameters ---------------------



  //------------------ Connecting to iRODS -----------------------------------

  conn = connect_to_irods(irodsParsed);

  //------------------ Finished Connecting to iRODS --------------------------



  //------------------ Transfering files to irods ----------------------------


 
//---
    convertCharStr(targPath.outPath, irodsParsed.dirname());
    rodsArgs.verbose = False;
    dataObjOprInp.oprType = PUT_OPR;
    setStateForRestart (conn, &rodsRestart, &targPath, &rodsArgs);
//===


  /*********************** Recursive Transfer ***************************/

  if(fileParsed.dirname() != NULL && irodsParsed.dirname() != NULL &&
      fileParsed.basename() == NULL && irodsParsed.basename() == NULL){

    // Perform recursive copy of the directory given in the source url

    FILE *infile;
    char path[MAX_STR];
    char dest[MAX_STR];
    char cmd[MAX_STR];

//--- 
/*    
    rodsPath_t targPath;
    rodsArguments_t rodsArgs;    
    dataObjInp_t dataObjOprInp;

    memset(&targPath, 0, sizeof(rodsPath_t));
    memset(&rodsArgs, 0, sizeof(rodsArguments_t));
    memset(&dataObjOprInp, 0, sizeof(dataObjInp_t));


    convertCharStr(targPath.outPath, irodsParsed.dirname());
    rodsArgs.verbose = False;
    dataObjOprInp.oprType = PUT_OPR;

    setStateForRestart (conn, &rodsRestart, &targPath, &rodsArgs);   
*/
//===


    // Create required directories at iRODS 
    sprintf(cmd, "find %s -type d", fileParsed.dirname());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open: tmp-SRBfilelist\n");
      return -1;
    }
    while(fgets(path, MAX_STR, infile) != NULL){
      	  strcpy(path+strlen(path)-1,"\0");
      	  sprintf(dest, "%s%s", irodsParsed.dirname(), path+strlen(fileParsed.dirname()));

//---
	  if(checkPoint){
      		dataObjOprInp.createMode = S_IFDIR;
      		//dataObjOprInp->condInput = NULL;
      		//status = chkStateForResume (conn, rodsRestart, targChildPath, rodsArgs, childObjType, &dataObjOprInp->condInput, 1); 
      		status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, COLL_OBJ_T, &dataObjOprInp.condInput, 1);
     		if(status<0) 
			return status;
      		else if(status==0) 
			continue;
	  }
//it seems like the directory doesn't need to do the chkStateForResume, it is always set to 1.     
//=== 

      	  status = mkCollR(conn, "/", dest);
      	  if(status < 0){
        	fprintf(stderr, "Collection creation error, status = %d\n", status);
        	rodsLogError (LOG_ERROR, status,"", "");
        	return -1;
      	  }
    }
    fclose(infile);
    // Finished Creating required directories at iRODS 

    // List all files in directory recursively and transfer
    sprintf(cmd, "find %s -type f", fileParsed.dirname());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open: tmp-irodsfilelist\n");
      return -1;
    }
    while(fgets(path, MAX_STR, infile) != NULL){
      	  strcpy(path+strlen(path)-1,"\0");
          sprintf(dest, "%s%s", irodsParsed.dirname(), path+strlen(fileParsed.dirname()));
//---
      	  if(checkPoint){
      	 	dataObjOprInp.createMode = S_IFREG; 
      	 	status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, DATA_OBJ_T, &dataObjOprInp.condInput, 1);
      	 	if(status<0) 
		 	return status;
      		else if(status==0) 
		 	continue;
      	   }
//===
      	  // Call to transfer function
          //status = transfer_from_file_to_irods(conn, path, dest, filesizeVerify, checksumVerify, mymode, nthreads);
//---
	  status = transfer_from_file_to_irods(conn, path, dest, filesizeVerify, checksumVerify, 0, mymode, nthreads);
//===	
          if(status < 0){
             fprintf(stderr, "Error: Transfer file %s to irods %s failed\n", path, dest);
             return -1;
          }
//---
      	   else{
		if (checkPoint){
           		rodsRestart.curCnt ++;
           		status = writeRestartFile (&rodsRestart, dest);
		}
      	   }
//===
    }

    fclose(infile);
    // Finished Recursive Transfer files to iRODS

  } 
  /*********************** Wild Card Transfer ***************************/
  else if(fileParsed.basename() != NULL && irodsParsed.basename() == NULL &&
      strchr(fileParsed.basename(), '*') != NULL){

    // Transfer files using wildcards

    FILE *infile;
    char path[MAX_STR];
    char dest[MAX_STR];
    char cmd[MAX_STR];


    // Create required directories at iRODS 
    status = mkCollR(conn, "/", irodsParsed.dirname());
    if(status < 0){
      fprintf(stderr, "Collection creation error, status = %d\n", status);
      rodsLogError (LOG_ERROR, status,"", "");
      return -1;
    }

    sprintf(cmd, "find %s -maxdepth 1 -name \"%s\" -type f", fileParsed.dirname(), fileParsed.basename());
    infile = popen(cmd,"r");
    if(infile == NULL){
      fprintf(stderr, "Error: Unable to open pipe:\n");
      return -1;
    }

    while(fgets(path, MAX_STR, infile) != NULL){
      	  strcpy(path+strlen(path)-1,"\0");
      	  sprintf(dest, "%s%s", irodsParsed.dirname(), strrchr(path, '/'));
//---
	  if (checkPoint){
      	  	dataObjOprInp.createMode = S_IFREG;
      	  	status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, DATA_OBJ_T, &dataObjOprInp.condInput, 1);
      	  	if(status<0) 
			return status;
      	  	else if(status==0) 
			continue;
	   }
//===

      // Call to transfer function
      //status = transfer_from_file_to_irods(conn, path, dest, filesizeVerify, checksumVerify, mymode, nthreads);
//---
	  status = transfer_from_file_to_irods(conn, path, dest, filesizeVerify, checksumVerify, 0, mymode, nthreads);
//===
      	  if(status < 0){
        	fprintf(stderr, "Error: Transfer file %s to irods %s failed!\n", path, dest);
        	return -1;
      	  }
//---
      	  else{
		if (checkPoint){
           		rodsRestart.curCnt ++;
           		status = writeRestartFile (&rodsRestart, dest);
		}
      	  }
//===
    }

    fclose(infile);

  }
  /*********************** Single File Transfer ***************************/
  else if((fileParsed.basename() != NULL) &&
      (strchr(fileParsed.basename(), '*') == NULL)){
    // Perform a single file transfer from source to destination

    char dest[MAX_STR];
    int len = strlen(irodsParsed.path());

    strncpy(dest, irodsParsed.path(), len);

    // Create required directories at iRODS 
    status = mkCollR(conn, "/", irodsParsed.dirname());
    if(status < 0){
      fprintf(stderr, "Collection creation error, status = %d\n", status);
      rodsLogError (LOG_ERROR, status,"", "");
      return -1;
    }

    // Use source filename as destination filename if destination file name not
    // given
    if(irodsParsed.basename() == NULL){
      len = strlen(irodsParsed.path()) + strlen(fileParsed.basename());
      sprintf(dest, "%s/%s", irodsParsed.dirname(), fileParsed.basename());
    }

    // Call to transfer function
/*
    status = transfer_from_file_to_irods(conn, fileParsed.path(), dest, 
          filesizeVerify, checksumVerify, mymode, nthreads);
*/
//---
    status = transfer_from_file_to_irods(conn, fileParsed.path(), dest,
          filesizeVerify, checksumVerify, fileCKPT, mymode, nthreads);	
//===

    if(status < 0){
      fprintf(stderr, "Error: Transfer file %s to irods %s failed!\n", 
          fileParsed.path(), dest);
      return -1;
    }
  }
  else{

    fprintf(stderr, "Please check the submitted urls and then try again\n");
    fprintf(stderr, "src:%s dest:%s\n", fileParsed.urlNoPassword(), 
        irodsParsed.urlNoPassword());

    return -1;
  }

  //------------------ Finished Transfering files to irods -------------------

//---
	//unlink("stork_file.rescue");
	unlink(file_ckpt_str);
//===
	
  rcDisconnect (conn);

  return 0;

}



/* ============================================================================
 * Performing transfers from iRODS to local filesystem
 * Parse submitted URLs
 * Obtain Stork Parameters
 * Connect to SRB server
 * Perform transfers according to type
 *  - Recursive
 *  - wildcard
 *  - single data object to file
 *  - else print error message
 * ==========================================================================*/

int irods_to_file(char *src, char *dst, int nthreads){

  rcComm_t *conn;
  int status;

  int checksumVerify = 0, filesizeVerify = 0, mymode = 0700;
 
//--- 
  int checkPoint = 0;
  
  int fileCKPT = 0;

    //rodsPath_t *targPath;
    //rodsArguments_t *rodsArgs;    
    //dataObjInp_t *dataObjOprInp;

    rodsPath_t targPath;
    rodsArguments_t rodsArgs;
    dataObjInp_t dataObjOprInp;


    memset(&targPath, 0, sizeof(rodsPath_t));
    memset(&rodsArgs, 0, sizeof(rodsArguments_t));
    memset(&dataObjOprInp, 0, sizeof(dataObjInp_t));

    //targPath->outPath = irodsParsed.dirname();
/*
    convertCharStr(targPath.outPath, irodsParsed.dirname());
    rodsArgs.verbose = False;
    dataObjOprInp.oprType = PUT_OPR;

    setStateForRestart (conn, &rodsRestart, &targPath, &rodsArgs);
*/
//===


  //------------------ Parsing URLs -----------------------------------------

  // Stork parse source (iRODS) url
  Url irodsParsed(src);
  if(!irodsParsed.parsed() ) {
    fprintf(stderr, "parse src URL '%s' error\n",
        irodsParsed.urlNoPassword()
        );
    return -1;
  }

  //printf("test2...\n");

  // Stork parse the source (iRODS) url as SRB
  // iRODS has similar fields as SRB
  irodsParsed.parseSrb();
  if(!irodsParsed.parsedSrb() ) {
    fprintf(stderr, "parse dest URL SRB fields'%s' error\n",
        irodsParsed.urlNoPassword());
    return -1;
  }

  //printf("test3...\n");

  // Stork parse destination (local) url
  Url fileParsed(dst);
  if(!fileParsed.parsed() ) {
    fprintf(stderr, "parse dest URL '%s' error\n",
        fileParsed.urlNoPassword()
        );
    return -1;
  }
   
  //printf("test4...\n"); 

  if(fileParsed.path() == NULL || irodsParsed.path() == NULL ||
      irodsParsed.host() == NULL || irodsParsed.port() == NULL ||
      irodsParsed.srbUser() == NULL || irodsParsed.srbMdasDomain() == NULL ||
      irodsParsed.password() == NULL){
    fprintf(stderr, "Error: Some fields are missing. Please recheck your URL "\
      "and resubmit the job.\nsrc: %s\ndest: %s\n", irodsParsed.urlNoPassword(),
      fileParsed.urlNoPassword());
    return -1;
  }

  //------------------ Finished Parsing URLs ---------------------------------



  //------------------ Obtain Stork Parameters -------------------------------

  //set_stork_parameters(&filesizeVerify, &checksumVerify, &mymode, &nthreads);
  set_stork_parameters(&filesizeVerify, &checksumVerify, &checkPoint, &fileCKPT, &mymode, &nthreads);

  //------------------ Finish obtaining Stork Parameters ---------------------



  //------------------ Connecting to iRODS -----------------------------------
  
  //printf("test5...\n");
  conn = connect_to_irods(irodsParsed);

  //------------------ Finished Connecting to iRODS --------------------------



  //------------------ Transfering files to irods ----------------------------


//---
    convertCharStr(targPath.outPath, fileParsed.dirname());
    rodsArgs.verbose = False;
    //dataObjOprInp.oprType = GET_OPR; // might not need

    setStateForRestart (conn, &rodsRestart, &targPath, &rodsArgs);

    FILE *tmpfile = fopen("test_rodsRestart", "w+");
//===

 

  /*********************** Recursive Transfer ***************************/

  //printf("test6...\n");

  if(fileParsed.dirname() != NULL && irodsParsed.dirname() != NULL &&
      fileParsed.basename() == NULL && irodsParsed.basename() == NULL){
    // Perform recursive copy of the directory given in the source url

    genQueryInp_t genQueryInp;
    genQueryOut_t *genQueryOut = NULL;
    int continueInx;
    int status;
    int i;

    sqlResult_t *dataName, *collName;
    char *tmpDataName;
    char *tmpCollName;
    char src[MAX_STR];
    char dest[MAX_STR];

/*
    rodsArguments_t rodsArgs;
    rodsArgs.recursive = True;

    status = queryCollInCollReCur (conn, irodsParsed.dirname(), &rodsArgs, 
        &genQueryInp, &genQueryOut);

*/

  //printf("test7...\n");  
	
//---
   queryHandle_t queryHandle;
   queryHandle.conn = conn;
   queryHandle.connType = RC_COMM;
   queryHandle.querySpecColl = (funcPtr) rcQuerySpecColl;
   queryHandle.genQuery = (funcPtr) rcGenQuery;

/*
   if(queryHandle.conn!=NULL)
        printf("testing 1...\n");
   else
        printf("testing 2...\n");

   printf("irods parsed dir: %s\n", irodsParsed.dirname());
*/
   status = queryCollInColl(&queryHandle, irodsParsed.dirname(), RECUR_QUERY_FG, &genQueryInp, &genQueryOut);
   //printf("testing colls return status = %d\n", status);
//===



    if (status < 0 && status != CAT_NO_ROWS_FOUND) {
      fprintf(stderr, "Error: unable to query iRODS for collections "\
          "recursively\n");
      rodsLogError(LOG_ERROR, status, "", "");
      return -1;
    }

    while(status >= 0){
      if (genQueryOut == NULL) {
        fprintf(stderr, "Error: No results generated\n");
        break;
      }

      if ((collName = getSqlResultByInx (genQueryOut, COL_COLL_NAME)) == NULL) {
        fprintf(stderr, "Couldn't retrieve the collection names from the "\
            "specified collection\n");
        return -1;
      }

      for (i = 0;i < genQueryOut->rowCnt; i++) {
           tmpDataName = &collName->value[collName->len * i];
           int len = strlen(irodsParsed.dirname());
           char *temp = (tmpDataName + len + 1);
           if((len = strlen(temp)) > 0){
               sprintf(src, "%s/%s", fileParsed.dirname(), temp);
               //printf("%s\n", temp);

//---
		if (checkPoint){
               		//dataObjOprInp.createMode = S_IFDIR;
               		//dataObjOprInp->condInput = NULL;
               		//status = chkStateForResume (conn, rodsRestart, targChildPath, rodsArgs, childObjType, &dataObjOprInp->condInput, 1); 
               		status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, COLL_OBJ_T, &dataObjOprInp.condInput, 1);
               		fprintf(tmpfile,"rodsRestart->curCnt=%d    rodsRestart->doneCnt=%d\n",rodsRestart.curCnt, rodsRestart.doneCnt);
               		if(status<0) 
				return status;
               		else if(status==0) 
				continue;
		}
                //it seems like the directory doesn't need to do the chkStateForResume, it is always set to 1.     
//===
          	file_mkdir(src, mymode);
        }
      }

      continueInx = genQueryOut->continueInx;
      freeGenQueryOut (&genQueryOut);
      if (continueInx > 0) {
        genQueryInp.continueInx = continueInx;
        status =  rcGenQuery (conn, &genQueryInp, &genQueryOut);
      } else {
        break;
      } 
    }

    clearGenQueryInp(&genQueryInp);



    //status = queryDataObjInCollReCur (conn, irodsParsed.dirname(), &rodsArgs, &genQueryInp, &genQueryOut);
    status = queryDataObjInColl(&queryHandle, irodsParsed.dirname(), RECUR_QUERY_FG, &genQueryInp, &genQueryOut, NULL);
    //printf("testing dataObj return status = %d\n", status);

    if (status < 0 && status != CAT_NO_ROWS_FOUND) {
      fprintf(stderr, "Error: Unable to retrieve filenames in directory\n");
      rodsLogError(LOG_ERROR, status, "", "");
      return -1;
    }

    while(status >= 0){
      if (genQueryOut == NULL) {
        fprintf(stderr, "Error: No results generated\n");
        break;
      }

      if ((collName = getSqlResultByInx (genQueryOut, COL_COLL_NAME)) == NULL) {
        fprintf(stderr, "Couldn't retrieve the collection names from the "\
            "specified collection\n");
        return -1;
      }

      if ((dataName = getSqlResultByInx (genQueryOut, COL_DATA_NAME)) == NULL) {
        fprintf(stderr, "Couldn't retrieve the data names from the "\
            "specified collection\n");
        return -1;
      }

      for (i = 0;i < genQueryOut->rowCnt; i++) {
        tmpCollName = &collName->value[collName->len * i];
        tmpDataName = &dataName->value[dataName->len * i];

        //char *temp = (tmpCollName + strlen(irodsParsed.dirname()) + 1); 
	char *temp = tmpCollName + strlen(irodsParsed.dirname()) ; 
        // temp = "" if there is no subdirectories        

        //if(strcmp(temp, "")==0) printf("temp is ' ' str\n");
        //else printf("temp is = %s\n", temp);

        sprintf(src,"%s/%s", tmpCollName, tmpDataName);

        //if(strcmp(temp, " ")==0){
        if(strlen(temp)==1 || strlen(temp)==0){
           //printf("temp is ' ' str\n");
           sprintf(dest,"%s/%s", fileParsed.dirname(), tmpDataName);
	}
        else{
           //printf("temp is %s templen= %d\n", temp, strlen(temp)); // sometimes strlen(temp) ==0, why?
           sprintf(dest,"%s%s/%s", fileParsed.dirname(), temp, tmpDataName);
	}
 
        //printf("src=%s\ndesc=%s\ntmpDataName=%s\ntmpCollName=%s\ntemp=%s\nirodsParsed.dirname()=%s\n", 
               //src, dst, tmpDataName, tmpCollName, temp, irodsParsed.dirname());
//--- 	
	if (checkPoint) {
      		//dataObjOprInp.createMode = S_IFREG;
      		status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, DATA_OBJ_T, &dataObjOprInp.condInput, 1);
      		fprintf(tmpfile,"rodsRestart->curCnt=%d    rodsRestart->doneCnt=%d\n",rodsRestart.curCnt, rodsRestart.doneCnt);
      		if(status<0) 
			return status;
      		else if(status==0) 
			continue;
	}
//===
        // Call to transfer function
        //status = transfer_from_irods_to_file(conn, src, dest, filesizeVerify, checksumVerify, mymode, nthreads);
//---
	status = transfer_from_irods_to_file(conn, src, dest, filesizeVerify, checksumVerify, 0, mymode, nthreads);
//===
        if(status < 0){
           fprintf(stderr, "Error: Transfer irods %s to file %s failed!\n", irodsParsed.path(), dest);
           return -1;
        }
//---
        else{
	      if(checkPoint){
           	rodsRestart.curCnt ++;
           	status = writeRestartFile (&rodsRestart, dest);
           	//procAndWrriteRestartFile (&rodsRestart, dest);
	      }	
        }
//===
      } //for

      continueInx = genQueryOut->continueInx;
      freeGenQueryOut (&genQueryOut);
      if (continueInx > 0) {
        genQueryInp.continueInx = continueInx;

        status =  rcGenQuery (conn, &genQueryInp, &genQueryOut);
      } else {
        	break;
      } 
    } //while

    clearGenQueryInp(&genQueryInp);
   
    fclose(tmpfile);

  } 
  /*********************** Wild Card Transfer ***************************/
  else if(irodsParsed.basename() != NULL && fileParsed.basename() == NULL &&
      strchr(irodsParsed.basename(), '*') != NULL){

    // Transfer files using wildcards
    /*
    genQueryInp_t genQueryInp;
    genQueryOut_t *genQueryOut = NULL;
    int continueInx;
    int status;
    int i;

    sqlResult_t *dataName, *collName;
    char *tmpDataName;
    char *tmpCollName;
    char src[MAX_STR];
    char dest[MAX_STR];
    char data0Cond[MAX_NAME_LEN];
    rodsArguments_t rodsArgs;



   char temp[56];
   char *tmp;
   strcpy(temp, irodsParsed.basename());
   tmp = strrchr(temp, '*');
   *tmp = '\0';
   //snprintf (data0Cond, MAX_NAME_LEN, " like '%s%%'", temp);
   snprintf (data0Cond, MAX_NAME_LEN, " like '%s'", temp);
   printf("data0Cond: %s\n",data0Cond);

   //addInxVal (&genQueryInp.sqlCondInp, COL_DATA_NAME, data0Cond);
   addInxVal (&genQueryInp.sqlCondInp, DATA_NAME, data0Cond);
   //addInxIval (&genQueryInp.selectInp, COL_DATA_NAME, 1);



    //status = queryDataObjInCollReCur (conn, irodsParsed.dirname(), &rodsArgs, 
    status = queryDataObjInColl(conn, irodsParsed.dirname(), &rodsArgs, 
        &genQueryInp, &genQueryOut);

    if (status < 0 && status != CAT_NO_ROWS_FOUND) {
      printf("1no rows\n");
    }

    while(status >= 0){
      if (genQueryOut == NULL) {
        printf("error:1\n");
      }

      if ((collName = getSqlResultByInx (genQueryOut, COL_COLL_NAME)) == NULL) {
        printf("error:3\n");
      }

      if ((dataName = getSqlResultByInx (genQueryOut, COL_DATA_NAME)) == NULL) {
        printf("error:3\n");
      }

      for (i = 0;i < genQueryOut->rowCnt; i++) {
        tmpCollName = &collName->value[collName->len * i];
        tmpDataName = &dataName->value[dataName->len * i];

        char *temp = (tmpCollName + strlen(irodsParsed.dirname()) + 1);

        sprintf(src,"%s/%s", tmpCollName, tmpDataName);
        sprintf(dest,"%s/%s/%s", fileParsed.dirname(), temp, tmpDataName);
        printf("%s %s\n", src, dst);
        
        // Call to transfer function
//        status = transfer_from_irods_to_file(conn, src, dest, filesize, 
//            checksum, mymode, nthreads);
//        if(status < 0){
//          fprintf(stderr, "Error: Transfer irods %s to file %s failed!\n", 
//              irodsParsed.path(), dest);
//          return -1;
//        }
      }

      continueInx = genQueryOut->continueInx;
      freeGenQueryOut (&genQueryOut);
      if (continueInx > 0) {
        genQueryInp.continueInx = continueInx;
        status =  rcGenQuery (conn, &genQueryInp, &genQueryOut);
      } else {
        break;
      } 
    }

    clearGenQueryInp(&genQueryInp);
    */
//---

    genQueryInp_t genQueryInp;

    genQueryOut_t *genQueryOut = NULL;
    int continueInx;
    //int status;
    int i;

    sqlResult_t *dataName, *collName;
    char *tmpDataName;
    char *tmpCollName;
    char src[MAX_STR];
    char dest[MAX_STR];

    char *filename, *file;
    
    char collQCond[MAX_NAME_LEN]; 
    //char collQCond2[MAX_NAME_LEN]; 
    int  status;

    //memset(&genQueryInp, 0, sizeof(genQueryInp_t));	

    if ((filename = (char *)malloc(strlen(irodsParsed.basename())*sizeof(char))) == NULL){
	 fprintf(stderr, "Error: memory allocaiton error for filename\n");
	 return -1;
    }
    strcpy(filename, irodsParsed.basename());
   
    //SQL database use % rather than * as wildcard
    while((file = strchr(filename, '*'))!=NULL){
	*file = '%';
        file = strchr(filename, '*');
    }


    snprintf(collQCond, MAX_NAME_LEN, " = '%s'", irodsParsed.dirname());
    addInxVal(&genQueryInp.sqlCondInp, COL_COLL_NAME, collQCond);    

    snprintf(collQCond, MAX_NAME_LEN, " like '%s'", filename);
    addInxVal(&genQueryInp.sqlCondInp, COL_DATA_NAME, collQCond);

    //printf("filename = %s collQCond = %s\n", filename, collQCond);
    setQueryInpForData(0, &genQueryInp);	

    genQueryInp.maxRows = MAX_SQL_ROWS;
    genQueryInp.options = RETURN_TOTAL_ROW_COUNT;

    status = rcGenQuery (conn, &genQueryInp, &genQueryOut);
	
    if (status < 0 && status != CAT_NO_ROWS_FOUND) {
      fprintf(stderr, "Error: Unable to retrieve filenames in directory\n");
      //fprintf(stderr, "status is = %d\n", status);
      return -1;
    }

    while(status >= 0){
     	if (genQueryOut == NULL) {
        	fprintf(stderr, "Error: No results generated\n");
        	break;
      	}

        if ((collName = getSqlResultByInx (genQueryOut, COL_COLL_NAME)) == NULL) {
          	fprintf(stderr, "Couldn't retrieve the collection names from the "\
            	"specified collection\n");
        	return -1;
      	}	

      	if ((dataName = getSqlResultByInx (genQueryOut, COL_DATA_NAME)) == NULL) {
        	fprintf(stderr, "Couldn't retrieve the data names from the "\
            	"specified collection\n");
        	return -1;
      	}
	
	for (i = 0;i < genQueryOut->rowCnt; i++) {
             	tmpCollName = &collName->value[collName->len * i];
             	tmpDataName = &dataName->value[dataName->len * i];
		
		sprintf(src,"%s/%s", tmpCollName, tmpDataName);
		sprintf(dest,"%s/%s", fileParsed.dirname(), tmpDataName);
	
//---
        	if (checkPoint) {
                	//dataObjOprInp.createMode = S_IFREG;
                	status = chkStateForResume (conn, &rodsRestart, dest, &rodsArgs, DATA_OBJ_T, &dataObjOprInp.condInput, 1);
                	if(status<0)
                        	return status;
                	else if(status==0)
                        	continue;
        	}
//===	
		status = transfer_from_irods_to_file(conn, src, dest, filesizeVerify, checksumVerify, 0, mymode, nthreads);
		 
		if(status < 0){
           		fprintf(stderr, "Error: Transfer irods %s to file %s failed!\n", irodsParsed.path(), dest);
           		return -1;
        	}
//---
        	else{
              		if(checkPoint){
                		rodsRestart.curCnt ++;
                		status = writeRestartFile (&rodsRestart, dest);
                	//procAndWrriteRestartFile (&rodsRestart, dest);
              		}
        	}
//===
	}

      	continueInx = genQueryOut->continueInx;
      	freeGenQueryOut (&genQueryOut);
      	if (continueInx > 0) {
        	genQueryInp.continueInx = continueInx;
        	status =  rcGenQuery (conn, &genQueryInp, &genQueryOut);
      	} else {
                break;
      	}

    } // while
	
    clearGenQueryInp(&genQueryInp);
//===

  } 
  /*********************** Single File Transfer ***************************/
  else if((irodsParsed.basename() != NULL) &&
          (strchr(irodsParsed.basename(), '*') == NULL)){
    // Perform a single file transfer from source to destination
   
    char dest[MAX_STR];
    int len = strlen(fileParsed.path());

    strncpy(dest, fileParsed.path(), len);

    // Create required directories at local file system 
    status = file_mkdir(fileParsed.dirname(), mymode);
    if(status < 0){
      fprintf(stderr, "Collection creation error, status = %d\n", status);
      return -1;
    }

    // Use source filename as destination filename
    if(fileParsed.basename() == NULL){
      len = strlen(fileParsed.path()) + strlen(irodsParsed.basename());
      sprintf(dest, "%s/%s", fileParsed.dirname(), irodsParsed.basename());
    }
      
    // Call to transfer function
    //status = transfer_from_irods_to_file(conn, irodsParsed.path(), dest, filesizeVerify, checksumVerify, mymode, nthreads);
//---
	status = transfer_from_irods_to_file(conn, irodsParsed.path(), dest, filesizeVerify, checksumVerify, fileCKPT, mymode, nthreads);
//===

    if(status < 0){
      fprintf(stderr, "Error: Transfer irods %s to file %s failed!\n", 
          irodsParsed.path(), dest);
      return -1;
    }
  }
  else{
   	 fprintf(stderr, "Please check the submitted urls and then try again\n");
    	 fprintf(stderr, "src:%s dest:%s\n", irodsParsed.urlNoPassword(), fileParsed.urlNoPassword());

    	 return -1;
  }

  //------------------ Finished Transfering files to irods -------------------

//---
	//unlink("stork_file.rescue");
	unlink(file_ckpt_str);
//===

  rcDisconnect (conn);


  return 0;
}

//----------------------- MAIN BODY --------------------------------------

int main(int argc, char **argv) {

  int status;
  int nthreads = -1;

//---
    struct timeval tv1, tv2;
    struct timezone tz1, tz2;

    gettimeofday (&tv1, &tz1); //started	
//===

  if(argc < 3){
    fprintf(stderr,
        "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url>\n", argv[0]); fprintf(stderr,
        "==============================================================\n");
    return DAP_ERROR;
  }
  
  for(status = 0;status < argc;status++){
    if(strncmp(argv[status], "-p", 2) == 0){
      nthreads = atoi(argv[++status]);
      break;
    }
  }

  if(!strncmp(argv[1],"file:/", 6) && !strncmp(argv[2], "irods://",8)){

    status = file_to_irods(argv[1], argv[2], nthreads);
    if(status < 0){
      fprintf(stderr, "Error: couldnot transfer file to irods\n");
      return DAP_ERROR;
    }

  }else if(!strncmp(argv[1], "irods://",8) && !strncmp(argv[2], "file:/", 6)){

    status = irods_to_file(argv[1], argv[2], nthreads);
    if(status < 0){
      fprintf(stderr, "Error: couldnot transfer irods to file\n");
      return DAP_ERROR;
    }

  }else{
    printf("Please check the syntax of the iRODS jobs submitted.\n");
    return DAP_ERROR;
  }

//---
    gettimeofday (&tv2, &tz2); //started
/*
    printf("checkpointing once start time:%d %d, end time %d %d, spend time %d sec | %d usec\n",
           tv1.tv_sec, tv1.tv_usec, tv2.tv_sec, tv2.tv_usec, (tv2.tv_sec - tv1.tv_sec), (tv2.tv_usec-tv1.tv_usec));
*/
    FILE * file_test = fopen("record_test.txt", "a+");
    fprintf(file_test, "start time:%d sec %d usec, end time%d sec %d usec, spend time (%d sec, %d usec)\n\n",
			tv1.tv_sec, tv1.tv_usec, tv2.tv_sec, tv2.tv_usec, (tv2.tv_sec - tv1.tv_sec), (tv2.tv_usec - tv1.tv_usec)); 
   fclose(file_test);
//===

  return DAP_SUCCESS;

} 
